@@ -286,8 +286,11 @@ vobjtovarid4 <- function( nc, varid, verbose=FALSE, allowdimvar=TRUE) {
 			if( class(varid) != 'ncid4' )
 				stop(paste("Internal error #D, returned varid is not a object of class ncid4"))
 			if( verbose )
-				print(paste("vobjtovarid4: returning with DIMvarid deduced from name; varid=",
-					varid$group_id, varid$id))
+				print(paste("vobjtovarid4: returning with DIMvarid deduced from name; varid$group_id=",
+					varid$group_id, 'varid$id=', varid$id))
+			if( varid$id == -1 ) 
+				print(paste("vobjtovarid4: **** WARNING **** I was asked to get a varid for dimension named",
+					origvarid, "BUT this dimension HAS NO DIMVAR! Code will probably fail at this point"))
 			return(varid)	# an object of class 'ncid4'
 			}
 		}
@@ -596,7 +599,12 @@ ncvar_id_hier <- function( ncid, varname ) {
 # (or NA if the variable does not have such an attribute).  Any
 # read-in values that are equal to missval are set to NA.
 #
-ncvar_get_inner <- function( ncid, varid, missval, addOffset=0., scaleFact=1.0, start=NA, count=NA, verbose=FALSE, signedbyte=TRUE, collapse_degen=TRUE ) {
+# if raw_datavals==TRUE, then the actual raw data values from the
+# file are returned with no conversion to NA (if missing) or 
+# scale/offset applied
+#
+ncvar_get_inner <- function( ncid, varid, missval, addOffset=0., scaleFact=1.0, start=NA, count=NA, verbose=FALSE, signedbyte=TRUE, 
+		collapse_degen=TRUE, raw_datavals=FALSE ) {
 
 	if( ! is.numeric(ncid))
 		stop("Error, first arg passed to ncvar_get_inner (ncid) must be a simple C-style integer that is passed directly to the C api")
@@ -730,7 +738,13 @@ ncvar_get_inner <- function( ncid, varid, missval, addOffset=0., scaleFact=1.0,
 			passed_missval = missval
 			imvstate = as.integer(2)
 			}
-		fixmiss = as.integer(1)
+
+		if( raw_datavals )
+			fixmiss = as.integer(0)	# setting 'raw_datavalues' to TRUE gives the actual raw numbers from the file, not processed at all
+		else
+			fixmiss = as.integer(1)
+
+		if( verbose ) print('about to call Rsx_nc4_get_vara_double...')
 		rv <- .Call("Rsx_nc4_get_vara_double", 
 			as.integer(ncid),
 			as.integer(varid),
@@ -742,6 +756,7 @@ ncvar_get_inner <- function( ncid, varid, missval, addOffset=0., scaleFact=1.0,
 			PACKAGE="ncdf4")
 		if( rv$error != 0 ) 
 			stop("C function R_nc4_get_vara_double returned error")
+		if( verbose ) print('back from call to Rsx_nc4_get_vara_double...')
 		data = rv$data
 		}
 
@@ -832,7 +847,9 @@ ncvar_get_inner <- function( ncid, varid, missval, addOffset=0., scaleFact=1.0,
 	#--------------------------------------------------------
 	# Set our dims...but collapse degenerate dimensions first
 	#--------------------------------------------------------
-	if( ndims > 0 ) {
+	if( verbose ) print(paste("ncvar_get_inner: will collapse degen dims if ndims > 0. ndims=", ndims, 
+			' collapse_degen=', collapse_degen, ' precint=', precint ))
+	if( (ndims > 0) && (length(rv$data) > 0)) {
 		if( collapse_degen ) {
 			count.nodegen <- vector()
 			foundone <- 0
@@ -851,7 +868,7 @@ ncvar_get_inner <- function( ncid, varid, missval, addOffset=0., scaleFact=1.0,
 					dim(rv$data) <- count.nodegen
 				}
 			}
-		else
+		else if( precint != 5 ) 
 			dim(rv$data) = count
 
 		if( verbose ) {
@@ -867,7 +884,8 @@ ncvar_get_inner <- function( ncid, varid, missval, addOffset=0., scaleFact=1.0,
 	# NOTE: if type is 3 or 4 (float or double), the missing
 	# value was already set by the C routine.
 	#----------------------------------------------------------
-	if( (precint != 5) && (precint != 3) && (precint != 4) ) {	# not char, float, or double
+	if( verbose ) print(paste("ncvar_get_inner: will now consider changing missing values to NA..."))
+	if( (!raw_datavals) && (precint != 5) && (precint != 3) && (precint != 4) ) {	# not char, float, or double
 		if( verbose ) print("ncvar_get: setting missing values to NA")
 		if( (precint==1) || (precint==2) || (precint==6) || (precint==7) || (precint==8) || (precint==9)) {
 			#--------------------------------------
@@ -893,10 +911,12 @@ ncvar_get_inner <- function( ncid, varid, missval, addOffset=0., scaleFact=1.0,
 	#--------------------------------------
 	# Implement add_offset and scale_factor
 	#--------------------------------------
-	if( (scaleFact != 1.0) || (addOffset != 0.0) ) {
-		if( verbose ) 
-			print(paste("ncvar_get: implementing add_offset=", addOffset, " and scaleFact=", scaleFact ))
-		rv$data <- rv$data * scaleFact + addOffset
+	if( ! raw_datavals ) {
+		if( (scaleFact != 1.0) || (addOffset != 0.0) ) {
+			if( verbose ) 
+				print(paste("ncvar_get: implementing add_offset=", addOffset, " and scaleFact=", scaleFact ))
+			rv$data <- rv$data * scaleFact + addOffset
+			}
 		}
 
 	return(rv$data)
